<?php

/**
 * Sch Mysql Class
 *
 * <code>
 * <?php
 * $mysql = new SchMysql();
 *
 * $sql = "SELECT * FROM `user` LIMIT 10";
 * $data = $mysql->getData( $sql );
 * $name = strip_tags( $_REQUEST['name'] );
 * $age = intval( $_REQUEST['age'] );
 * $sql = "INSERT  INTO `user` ( `name` , `age` , `regtime` ) VALUES ( '"  . $mysql->escape( $name ) . "' , '" . intval( $age ) . "' , NOW() ) ";
 * $mysql->runSql( $sql );
 * if( $mysql->errno() != 0 )
 * {
 *    die( "Error:" . $mysql->errmsg() );
 * }
 *
 * $mysql->closeDb();
 * ?>
 * </code>
 *
 *
 */
class SchMysql
{
    private $host;
    private $port;
    private $username;
    private $password;
    private $db_name;
    private $charset;

    /**
     * 构造函数
     * SchMysql constructor.
     * @param $host
     * @param $port
     * @param $username
     * @param $password
     * @param $db_name
     * @param string $charset
     */
    function __construct($host, $port, $username, $password, $db_name, $charset = 'utf8')
    {
        $this->host = $host;
        $this->port = $port;
        $this->username = $username;
        $this->password = $password;
        $this->db_name = $db_name;
        $this->charset = $charset;
    }

    /**
     * 重新设置用户名和密码
     * @param $username
     * @param $password
     */
    public function setAuth($username, $password)
    {
        $this->username = $username;
        $this->password = $password;
    }


    /**
     * 设置主机和端口
     * @param $host
     * @param $port
     */
    public function setHostAndPort($host, $port)
    {
        $this->port = intval($port);
        $this->host = $host;

    }

    /**
     * 设置数据库名称
     * @param $db_name
     */
    public function setDbName($db_name)
    {
        $this->db_name = $db_name;
    }


    /**
     * 设置当前连接的字符集 , 必须在发起连接之前进行设置
     *
     * @param string $charset 字符集,如GBK,GB2312,UTF8
     * @return void
     */
    public function setCharset($charset)
    {
        return $this->set_charset($charset);
    }

    /**
     * 同setCharset,向前兼容
     *
     * @param string $charset
     * @return void
     * @ignore
     */
    public function set_charset($charset)
    {
        $this->charset = $charset;
    }

    /**
     * 运行Sql语句,不返回结果集
     *
     * @param string $sql
     * @return mysqli_result|bool
     */
    public function runSql($sql)
    {
        return $this->run_sql($sql);
    }

    /**
     * 同runSql,向前兼容
     *
     * @param string $sql
     * @return bool
     * @ignore
     */
    public function run_sql($sql)
    {
        $this->last_sql = $sql;
        $db_link = $this->db_write();
        if ($db_link === false) {
            return false;
        }
        $ret = mysqli_query($db_link, $sql);
        $this->save_error($db_link);
        return $ret;
    }

    /**
     * 运行Sql,以多维数组方式返回结果集
     *
     * @param string $sql
     * @return array 成功返回数组，失败时返回false
     */
    public function getData($sql)
    {
        return $this->get_data($sql);
    }

    /**
     * 同getData,向前兼容
     *
     * @ignore
     */
    public function get_data($sql)
    {
        $this->last_sql = $sql;
        $data = Array();
        $i = 0;
        $db_link = $this->do_replication ? $this->db_read() : $this->db_write();
        if ($db_link === false) {
            return false;
        }
        $result = mysqli_query($db_link, $sql);

        $this->save_error($db_link);

        if (is_bool($result)) {
            return $result;
        } else {
            while ($Array = mysqli_fetch_array($result, MYSQLI_ASSOC)) {
                $data[$i++] = $Array;
            }
        }

        mysqli_free_result($result);

        if (count($data) > 0)
            return $data;
        else
            return NULL;
    }

    /**
     * 运行Sql,以数组方式返回结果集第一条记录
     *
     * @param string $sql
     * @return array 成功返回数组，失败时返回false
     */
    public function getLine($sql)
    {
        return $this->get_line($sql);
    }

    /**
     * 同getLine,向前兼容
     *
     * @param string $sql
     * @return array
     * @ignore
     */
    public function get_line($sql)
    {
        $data = $this->get_data($sql);
        if ($data) {
            return @reset($data);
        } else {
            return false;
        }
    }

    /**
     * 运行Sql,返回结果集第一条记录的第一个字段值
     *
     * @param string $sql
     * @return mixed 成功时返回一个值，失败时返回false
     */
    public function getVar($sql)
    {
        return $this->get_var($sql);
    }

    /**
     * 同getVar,向前兼容
     *
     * @param string $sql
     * @return array
     * @ignore
     */
    public function get_var($sql)
    {
        $data = $this->get_line($sql);
        if ($data) {
            return $data[@reset(@array_keys($data))];
        } else {
            return false;
        }
    }

    /**
     * 同mysqli_affected_rows函数
     *
     * @return int 成功返回行数,失败时返回-1
     */
    public function affectedRows()
    {
        $result = isset($this->db_write) ? mysqli_affected_rows($this->db_write) : -1;
        return $result;
    }

    /**
     * 同mysqli_insert_id函数
     *
     * @return int 成功返回last_id,失败时返回false
     */
    public function lastId()
    {
        return $this->last_id();
    }

    /**
     * 同lastId,向前兼容
     *
     * @return int
     * @ignore
     */
    public function last_id()
    {
        $result = mysqli_insert_id($this->db_write(false));
        return $result;
    }

    /**
     * 关闭数据库连接
     *
     * @return void
     */
    public function closeDb()
    {
        return $this->close_db();
    }

    /**
     * 同closeDb,向前兼容
     *
     * @return void
     * @ignore
     */
    public function close_db()
    {
        if (isset($this->db_read))
            @mysqli_close($this->db_read);

        if (isset($this->db_write))
            @mysqli_close($this->db_write);

    }

    /**
     *  同mysqli_real_escape_string
     *
     * @param string $str
     * @return string
     */
    public function escape($str)
    {
        if (isset($this->db_read)) {
            $db = $this->db_read;
        } elseif (isset($this->db_write)) {
            $db = $this->db_write;
        } else {
            $db = $this->db_read();
        }

        return mysqli_real_escape_string($db, $str);
    }

    /**
     * 返回错误码
     *
     *
     * @return int
     */
    public function errno()
    {
        return $this->errno;
    }

    /**
     * 返回错误信息
     *
     * @return string
     */
    public function error()
    {
        return $this->error;
    }

    /**
     * 返回错误信息,error的别名
     *
     * @return string
     */
    public function errmsg()
    {
        return $this->error();
    }

    /**
     * @ignore
     */
    private function connect()
    {
        if ($this->port == 0) {
            $this->error = 13048;
            $this->errno = 'Not Initialized';
            return false;
        }

        $db = mysqli_init();
        mysqli_options($db, MYSQLI_OPT_CONNECT_TIMEOUT, 5);

        if (!mysqli_real_connect($db, $this->host, $this->username, $this->password, $this->db_name, $this->port)) {
            $this->error = mysqli_connect_error();
            $this->errno = mysqli_connect_errno();
            return false;
        }

        mysqli_set_charset($db, $this->charset);

        return $db;
    }

    /**
     * @ignore
     */
    private function db_read()
    {
        if (isset($this->db_read) && mysqli_ping($this->db_read)) {
            return $this->db_read;
        } else {
            if (!$this->do_replication) return $this->db_write();
            else {
                $this->db_read = $this->connect(false);
                return $this->db_read;
            }
        }
    }

    /**
     * @ignore
     */
    private function db_write($reconnect = true)
    {
        if (isset($this->db_write) && ($reconnect == false || mysqli_ping($this->db_write))) {
            return $this->db_write;
        } else {
            $this->db_write = $this->connect(true);
            return $this->db_write;
        }
    }

    /**
     * @ignore
     */
    private function save_error($dblink)
    {
        $this->error = mysqli_error($dblink);
        $this->errno = mysqli_errno($dblink);
    }

    private $error;
    private $errno;
    private $last_sql;
}